/**
* Copyright (c) 2021-2022 Kedron Holdings LLC, All Rights Reserved.
* This file is not part of Reactor, but a free add-on for it. This file only is offered under the MIT License.
* See https://mit-license.org for license text/details.
*/

const version = 22280;

const Configuration = require("server/lib/Configuration");
const Logger = require("server/lib/Logger");
const Controller = require("server/lib/Controller").requires( 22125 );

Logger.getDefaultLogger().log( null, "Module OctoPrintController v%1", version );

const basedir = Configuration.getConfig( "reactor.basedir" );
const dumpdir = Configuration.getConfig( "reactor.logsdir" );
const confdir = Configuration.getConfig( "reactor.confdir" );

module.exports = class OctoPrintController extends Controller {
    constructor( struct, id, config ) {
        super( struct, id, config );

        this.num_fail = 0;

        /* Since this Controller only represents one thing, use the system entity to carry it. */
        this.removeControllerGroup();
        this.system.extendCapability( 'printer3d' );
    }

    /** Start the controller.
     */
    async start() {
        this.run();
        this.online();
        return this;
    }

    run() {
        const e = this.system;
        this.log.debug( 5, "%1 querying printer at %2", this, this.config.source );
        this.fetchJSON( this.config.source + '/api/printer',
            {
                timeout: this.config.timeout || 15000,
                headers: { 'x-api-key': this.config.api_key }
            }
        ).then( data => {
            this.num_fail = 0;
            this.log.debug( 5, "%1: received response data %2", this, data );
            e.deferNotifies( true );
            e.markDead( false );
            try {
                e.setAttribute( 'printer3d.reachable', true );
                e.setAttribute( 'printer3d.printing', data.state.flags.printing );
                e.setAttribute( 'printer3d.paused', data.state.flags.paused );
                e.setAttribute( 'printer3d.resuming', data.state.flags.resuming );
                e.setAttribute( 'printer3d.ready', data.state.flags.ready );
                e.setAttribute( 'printer3d.operational', data.state.flags.operational );
                e.setAttribute( 'printer3d.error', data.state.flags.error );
                e.setAttribute( 'printer3d.finishing', data.state.flags.finishing );
                e.setAttribute( 'printer3d.temperature_bed_actual', data.temperature.bed.actual );
                e.setAttribute( 'printer3d.temperature_bed_target', data.temperature.bed.target );
                e.setAttribute( 'printer3d.temperature_hotend_actual', data.temperature.tool0.actual );
                e.setAttribute( 'printer3d.temperature_hotend_target', data.temperature.tool0.target );
                e.setAttribute( 'printer3d.message', data.state.text );
                e.setAttribute( 'printer3d.sdcard_ready', data.sd.ready );
            } catch ( err ) {
                this.log.exception( err );
            } finally {
                e.deferNotifies( false );
                this.startDelay( this.config.interval || 60000 );
            }
        }).catch( err => {
            this.log.err( "%1: error fetching printer state: %2", this, err );
            this.startDelay( Math.min( 120000, ( this.config.error_interval || 5000 ) * Math.max( 0, ++this.num_fail - 12 ) ) );
            if ( this.num_fail >= 3 ) {
                e.setAttributes( {
                    reachable: false,
                    ready: false,
                    operational: false,
                    error: true,
                    message: `${this.config.source} ${String(err)}`
                }, 'printer3d' );
            }
        });
    }
};
